# -*- coding: utf-8 -*-

"""
/***************************************************************************
 GroupStats
                                 A QGIS plugin
 Calculates stats for group of features
                              -------------------
        begin                : 2011-04-25
        copyright            : (C) 2011 by Rajmund Szostok
        email                : rayo001@interia.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
from math import *
import os
import csv
# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from groupstatsdialog import GroupStatsDialog

class GroupStats:

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface
        
        #i18n
        pluginPath = QFileInfo(QgsApplication.qgisUserDbFilePath()).path()+"/python/plugins/GroupStats"
        localeName = QLocale.system().name()
        if QFileInfo(pluginPath).exists():
          self.localePath = pluginPath+"/i18n/groupstats_" + localeName + ".qm"
        if QFileInfo(self.localePath).exists():
          self.translator = QTranslator()
          self.translator.load(self.localePath)
          if qVersion() > '4.3.3':
            QCoreApplication.installTranslator(self.translator)  

    def initGui(self):
        # Create action that will start plugin configuration
        self.action = QAction(QIcon(":/plugins/groupstats/icon.png"), \
            "Group Stats", self.iface.mainWindow())
        # connect the action to the run method
        QObject.connect(self.action, SIGNAL("triggered()"), self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu("&Group Stats", self.action)
        


    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu("&Group Stats",self.action)
        self.iface.removeToolBarIcon(self.action)

    # run method that performs all the real work
    def run(self):

        self.okno = GroupStatsDialog()                             # Definicja okna
        
        warstwy = QgsMapLayerRegistry.instance().mapLayers()        # Wczytanie nazw warstw z projektu
        nazwyPosortowane = warstwy.keys()
        nazwyPosortowane.sort()
        
        w_wektorowe = 0
        for id in nazwyPosortowane:
            warstwa = warstwy[id]                                    # Wczytanie nazw warstw do okna
            if warstwa.type()==0:
                self.okno.ui.warstwy.addItem(warstwa.name(), id)
                w_wektorowe = w_wektorowe +1
        
        if w_wektorowe == 0 or len(warstwy) == 0:
            QMessageBox.information(None,QCoreApplication.translate('GroupStats','Information'), \
                QCoreApplication.translate('GroupStats','Vector layers not found'))
            return
                
        
        QObject.connect(self.okno.ui.warstwy, SIGNAL("currentIndexChanged (int)"), self.wyborWarstwy)   # Sygnał wyboru warstwy
        QObject.connect(self.okno.ui.tylkoZaznaczone, SIGNAL("stateChanged (int)"), self.zerujPostep)   # Sygnał zmiany 'Tylko wybrane obiekty'
        QObject.connect(self.okno.ui.filtr1_pole, SIGNAL("currentIndexChanged (int)"), self.zmianaFiltra1)   # Sygnał wyboru filtra
        QObject.connect(self.okno.ui.filtr1_warunek, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał wyboru filtra
        QObject.connect(self.okno.ui.filtr1_wartosc, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sprawdzic co z wpisaniem z reki do pola !!!!!!!!
        QObject.connect(self.okno.ui.filtry_operator1, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał wyboru filtra
        QObject.connect(self.okno.ui.filtr2_pole, SIGNAL("currentIndexChanged (int)"), self.zmianaFiltra2)   # Sygnał wyboru filtra
        QObject.connect(self.okno.ui.filtr2_warunek, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał wyboru filtra
        QObject.connect(self.okno.ui.filtr2_wartosc, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sprawdzic co z wpisaniem z reki do pola !!!!!!!!
        QObject.connect(self.okno.ui.poleWartosci, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał zmiany wartosci
        
        
        #QObject.connect(self.okno.ui.poleKlasyfikacji, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał zmiany klasyfikatora
        
        
        QObject.connect(self.okno.ui.radioAtrybuty, SIGNAL("toggled (bool)"), self.zmianaAtrybutyGeom)   # Sygnał zmiany atrybuty/geometria
        QObject.connect(self.okno.ui.radioGeometria, SIGNAL("toggled (bool)"), self.zmianaAtrybutyGeom)   # Sygnał zmiany atrybuty/geometria
        QObject.connect(self.okno.ui.kopiuj, SIGNAL("clicked ()"), self.kopiowanie)   # Sygnał kopiowania do schowka
        QObject.connect(self.okno.ui.zapisz, SIGNAL("clicked ()"), self.eksportCSV)   # Sygnał zapisu do pliku CSV
        QObject.connect(self.okno.ui.oblicz, SIGNAL("clicked ()"), self.obliczenia)                 # Sygnał rozpoczęcia obliczeń
 
        self.okno.show()                                            # Pokaż okno
        self.wyborWarstwy(0)                                        # Wybierz pierwszą warstwę
        
        
    def wyborWarstwy(self, indeks):
        
        id = self.okno.ui.warstwy.itemData(indeks)                          # Pobranie ID wybranej warstwy
        warstwa = QgsMapLayerRegistry.instance().mapLayer(id.toString())    
        provider = warstwa.dataProvider()
        atrybuty = provider.fields()                                        # Pobranie atrybutów wybranej warstwy
        
        self.okno.ui.filtr1_pole.blockSignals(True)                         # Zablokowanie sygnałów przed zmianami w polach
        self.okno.ui.filtr1_wartosc.blockSignals(True)
        self.okno.ui.filtr2_pole.blockSignals(True)                        
        self.okno.ui.filtr2_wartosc.blockSignals(True)
        self.okno.ui.poleWartosci.blockSignals(True)
        
        self.okno.ui.poleKlasyfikacji.clear()                               # Wyczyszczenie pól
        self.okno.ui.poleKlasyfikacji2.clear()
        self.okno.ui.poleKlasyfikacji3.clear()
        self.okno.ui.filtr1_pole.clear()
        self.okno.ui.filtr1_wartosc.clear()
        self.okno.ui.filtr2_pole.clear()
        self.okno.ui.filtr2_wartosc.clear()
        self.okno.ui.poleWartosci.clear()
        
        self.okno.ui.poleKlasyfikacji2.addItem('-',-1)                      # Dodanie pozycji oznaczającej brak wyboru
        self.okno.ui.poleKlasyfikacji3.addItem('-',-1)
        self.okno.ui.filtr1_pole.addItem('-',-1)
        self.okno.ui.filtr2_pole.addItem('-',-1)
        
        for id, atrybut in atrybuty.iteritems():                            # Dodanie pozycji z atrybutami do pól wyboru
            self.okno.ui.poleKlasyfikacji.addItem(atrybut.name(),id)
            self.okno.ui.poleKlasyfikacji2.addItem(atrybut.name(),id)
            self.okno.ui.poleKlasyfikacji3.addItem(atrybut.name(),id)
            self.okno.ui.filtr1_pole.addItem(atrybut.name(),id)
            self.okno.ui.filtr2_pole.addItem(atrybut.name(),id)
        
        if self.okno.ui.radioGeometria.isChecked():                         # Wypełnienie pola wartości dla geometrii
            if warstwa.geometryType()==0:                               # point
                self.okno.ui.radioAtrybuty.setChecked(True)
            elif warstwa.geometryType()==1:                             # line
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Length'))
            elif warstwa.geometryType()==2:                             # polygon
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Area'))
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Perimeter'))
        else:                                                               # lub dla atrybutów
            for id, atrybut in atrybuty.iteritems():                            
                if atrybut.typeName().toUpper() in ('REAL', 'INTEGER', 'INT4', 'INT8', 'FLOAT4'):
                    self.okno.ui.poleWartosci.addItem(atrybut.name(),id)        # Dodanie tylko pól numerycznych
        
        self.okno.ui.filtr1_pole.blockSignals(False)                            # Odblokowanie sygnałów
        self.okno.ui.filtr1_wartosc.blockSignals(False)
        self.okno.ui.filtr2_pole.blockSignals(False)                         
        self.okno.ui.filtr2_wartosc.blockSignals(False)
        self.okno.ui.poleWartosci.blockSignals(False)
        
        self.zerujPostep(1)                                                     # Wyczyszczenie tabeli i paska postępu



    def zmianaFiltra1(self, nic):
        
        # zablokowac sygnaly z pola wartosci filtra
        
        self.zerujPostep(1)                                                     # Wyczyszczenie tabeli i paska postępu
        
        idFiltr1 = self.okno.ui.filtr1_pole.currentIndex()
        filtr1 = self.okno.ui.filtr1_pole.itemData(idFiltr1).toInt()[0]

        if filtr1 <> -1:
        
            indeks = self.okno.ui.warstwy.currentIndex()                          # Pobranie ID wybranej warstwy
            id = self.okno.ui.warstwy.itemData(indeks)
            warstwa = QgsMapLayerRegistry.instance().mapLayer(id.toString())    
            provider = warstwa.dataProvider()
            
    
            
            wartosci = provider.uniqueValues(filtr1)                                        # Pobranie atrybutów wybranej warstwy
            
            self.okno.ui.filtr1_wartosc.clear()
            
            for atrybut in wartosci:
                self.okno.ui.filtr1_wartosc.addItem(atrybut.toString())
            
            #QMessageBox.information(None,"Uwaga!", \
            #    "%s" % (wartosci[0].toString()))

        else:
            
            self.okno.ui.filtr1_wartosc.clear()


    def zmianaFiltra2(self, nic):
        
        # zablokowac sygnaly z pola wartosci filtra
        
        self.zerujPostep(1)                                                     # Wyczyszczenie tabeli i paska postępu
        
        idFiltr2 = self.okno.ui.filtr2_pole.currentIndex()
        filtr2 = self.okno.ui.filtr2_pole.itemData(idFiltr2).toInt()[0]

        if filtr2 <> -1:
        
            indeks = self.okno.ui.warstwy.currentIndex()                          # Pobranie ID wybranej warstwy
            id = self.okno.ui.warstwy.itemData(indeks)
            warstwa = QgsMapLayerRegistry.instance().mapLayer(id.toString())    
            provider = warstwa.dataProvider()
            
    
            
            wartosci = provider.uniqueValues(filtr2)                                        # Pobranie atrybutów wybranej warstwy
            
            self.okno.ui.filtr2_wartosc.clear()
            
            for atrybut in wartosci:
                self.okno.ui.filtr2_wartosc.addItem(atrybut.toString())
            
            #QMessageBox.information(None,"Uwaga!", \
            #    "%s" % (wartosci[0].toString()))

        else:
            
            self.okno.ui.filtr2_wartosc.clear()



    def zmianaAtrybutyGeom(self, nic):
        
        
                # zablokowac sygnaly z pola wartosci 
        
        self.zerujPostep(1)    
        
        indeks = self.okno.ui.warstwy.currentIndex()
        id = self.okno.ui.warstwy.itemData(indeks)                          # Pobranie ID wybranej warstwy
        warstwa = QgsMapLayerRegistry.instance().mapLayer(id.toString())
        provider = warstwa.dataProvider()
        atrybuty = provider.fields()
        
        
        
        #klasyfikator1 = self.okno.ui.poleKlasyfikacji.currentIndex()
        #klasyfikator2 = self.okno.ui.poleKlasyfikacji2.currentIndex()
        #klasyfikator3 = self.okno.ui.poleKlasyfikacji3.currentIndex()
        #self.wyborWarstwy(indeks, klasyfikator1, klasyfikator2, klasyfikator3)
        if self.okno.ui.radioGeometria.isChecked():
            if warstwa.geometryType()==0:                               # point
                self.okno.ui.radioAtrybuty.setChecked(True)
            elif warstwa.geometryType()==1:                             # line
                self.okno.ui.poleWartosci.clear()
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Length'))
            elif warstwa.geometryType()==2:                             # polygon
                self.okno.ui.poleWartosci.clear()
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Area'))
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Perimeter'))
        else:
            self.okno.ui.poleWartosci.clear()
            for id, atrybut in atrybuty.iteritems():
                if atrybut.typeName().toUpper() in ('REAL', 'INTEGER', 'INT4', 'INT8', 'FLOAT4'):
                    self.okno.ui.poleWartosci.addItem(atrybut.name(),id)
    
    
    def obliczenia(self):
        
        self.okno.ui.wyniki.horizontalHeaderItem(0).setText( self.okno.ui.poleKlasyfikacji.currentText() )      # Nazwanie kolumn klas
        self.okno.ui.wyniki.horizontalHeaderItem(1).setText( self.okno.ui.poleKlasyfikacji2.currentText() )
        self.okno.ui.wyniki.horizontalHeaderItem(2).setText( self.okno.ui.poleKlasyfikacji3.currentText() )
        
        indeks = self.okno.ui.warstwy.currentIndex()                                        # Pobranie wybranej warstwy
        idWarstwy = self.okno.ui.warstwy.itemData(indeks)
        warstwa = QgsMapLayerRegistry.instance().mapLayer(idWarstwy.toString())
        
        if self.okno.ui.tylkoZaznaczone.isChecked():                                        # Pobranie ID zaznaczonych obiektów
            zaznaczoneObiektyIds = warstwa.selectedFeaturesIds()
            tylkoZaznaczone = True
        else:
            zaznaczoneObiektyIds = []
            tylkoZaznaczone = False
        
        
        idKlas1 = self.okno.ui.poleKlasyfikacji.currentIndex()                              # Pobranie ID klasyfikatorów klas
        klasyfikator1 = self.okno.ui.poleKlasyfikacji.itemData(idKlas1).toInt()[0]
        idKlas2 = self.okno.ui.poleKlasyfikacji2.currentIndex()
        klasyfikator2 = self.okno.ui.poleKlasyfikacji2.itemData(idKlas2).toInt()[0]
        idKlas3 = self.okno.ui.poleKlasyfikacji3.currentIndex()
        klasyfikator3 = self.okno.ui.poleKlasyfikacji3.itemData(idKlas3).toInt()[0]
        
        if klasyfikator2 == -1:                                                             # Klasyfikator nr 2
            atrybut2 = lambda a2: None                                                      # jeśli nie wybrany to pusty
        else:
            atrybut2 = lambda a2: a2[klasyfikator2].toString()                              # jeśli wybrany to wartość atrybutu danego obiektu
        
        if klasyfikator3 == -1:                                                             # Klasyfikator nr 3 j.w.
            atrybut3 = lambda a3: None
        else:
            atrybut3 = lambda a3: a3[klasyfikator3].toString()
        
        
        wartoscTxt = self.okno.ui.poleWartosci.currentText()                                # Pobranie danych pola wartości
        idWart = self.okno.ui.poleWartosci.currentIndex()
        wartosc = self.okno.ui.poleWartosci.itemData(idWart).toInt()[0]
        
        if self.okno.ui.radioGeometria.isChecked():                                         # Obliczanie wartości z geometrii
            if wartoscTxt == QCoreApplication.translate('GroupStats','Length') or wartoscTxt == QCoreApplication.translate('GroupStats','Perimeter'):
                wartosc_obiektu = lambda obiekt: obiekt.geometry().length()
            elif wartoscTxt == QCoreApplication.translate('GroupStats','Area'):
                wartosc_obiektu = lambda obiekt: obiekt.geometry().area()
        else:
                wartosc_obiektu = lambda obiekt: obiekt.attributeMap()[wartosc].toReal()[0] # Obliczanie wartości z atrybutów
        
        
        idFiltr1 = self.okno.ui.filtr1_pole.currentIndex()                                  # Dane filtra nr 1
        filtr1 = self.okno.ui.filtr1_pole.itemData(idFiltr1).toInt()[0]                     # Pobranie ID dla atrybutu filtra
        filtr11 = self.okno.ui.filtr1_warunek.currentText()                                 # Pobranie operatora filtra
        filtr111 = self.okno.ui.filtr1_wartosc.currentText()                                # Pobranie wartości filtra
        
        idFiltr2 = self.okno.ui.filtr2_pole.currentIndex()                                  # Dane filtra nr 1
        filtr2 = self.okno.ui.filtr2_pole.itemData(idFiltr2).toInt()[0]                     # Pobranie ID dla atrybutu filtra
        filtr22 = self.okno.ui.filtr2_warunek.currentText()                                 # Pobranie operatora filtra
        filtr222 = self.okno.ui.filtr2_wartosc.currentText()                                # Pobranie wartości filtra
        
        filtry_operator1 = self.okno.ui.filtry_operator1.currentText()
        
        
        #if filtr1 == -1:                                                                    # Jeżeli filtr nie wybrany to nie filtruj
        #    filtr1_spr = lambda: True
        #else:
        #    filtr1_spr = lambda: eval("filtr10 %s filtr111" % (filtr11))                 # jeśli wybrany to sprawdzaj warunek
        
        
        #QMessageBox.information(None,"Uwaga!", \
        #    "%s, %s, %s" % (filtr1, filtr11, filtr111))
        
        
        provider = warstwa.dataProvider()
        atrybutyWszy = provider.attributeIndexes()                                          # Wybranie wszystkich obiektów
        provider.select(atrybutyWszy)
        f=QgsFeature()
        wyniki = {}                                                                         # Słownik na wyniki
        
        licznik_caly = provider.featureCount()                                              # Licznik do paska postępu
        licznik = licznik_caly
        
        
        while provider.nextFeature(f):                                                      # Wyszukiwanie właściwe
            if filtr1 <> -1:                                                                # Zmienna do filtrowania
                filtr10 = f.attributeMap()[filtr1].toString()
                filtr1_spr = eval("filtr10 %s filtr111" % (filtr11))
            #else:
             #   filtr1_spr = True

            if filtr2 <> -1:                                                                # Zmienna do filtrowania
                filtr20 = f.attributeMap()[filtr2].toString()
                filtr2_spr = eval("filtr20 %s filtr222" % (filtr22))
            #else:
             #   filtr2_spr = True
                
            if filtr1 <> -1 and filtr2 <> -1:                                                                # Zmienna do filtrowania
                filtr_spr = eval("filtr1_spr %s filtr2_spr" % (filtry_operator1))
            elif filtr1 <> -1 and filtr2 == -1:
                filtr_spr = filtr1_spr
            elif filtr1 == -1 and filtr2 <> -1:
                filtr_spr = filtr2_spr
            else:
                filtr_spr = True
                
            if ((tylkoZaznaczone==False) or (tylkoZaznaczone and (f.id() in zaznaczoneObiektyIds))) and filtr_spr:
                if wyniki.has_key((f.attributeMap()[klasyfikator1].toString(), atrybut2(f.attributeMap()), atrybut3(f.attributeMap()))):
                    wyniki[f.attributeMap()[klasyfikator1].toString(), atrybut2(f.attributeMap()), atrybut3(f.attributeMap())].append(wartosc_obiektu(f))
                else:
                    wyniki[f.attributeMap()[klasyfikator1].toString(), atrybut2(f.attributeMap()), atrybut3(f.attributeMap())] = [wartosc_obiektu(f)]
            licznik = licznik-1
            self.okno.ui.postep.setValue(int(100*(licznik_caly-licznik)/licznik_caly))


     
        
        for n in range(self.okno.ui.wyniki.rowCount(),-1,-1):      # Wyczyść tabelę wyników
            self.okno.ui.wyniki.removeRow(n)
        
        self.okno.ui.wyniki.setSortingEnabled (0)
        
        for grupa, wartosci in wyniki.iteritems():                                         # Wpisz wyniki do tabeli
            
            liczebnosc = len(wyniki[grupa])
            suma = sum(wyniki[grupa])
            srednia = suma/liczebnosc
            
            wariancja = 0
            for x in wyniki[grupa]:
                wariancja = wariancja + (x-srednia)**2
            wariancja = wariancja/liczebnosc
            
            odchylenie = sqrt(wariancja)
            
            wyniki[grupa].sort()
            if liczebnosc == 1:
                mediana = wyniki[grupa][0]
            else:
                pozycja = liczebnosc/2
                if liczebnosc%2 == 0:
                    mediana = (wyniki[grupa][pozycja]+wyniki[grupa][pozycja-1])/2
                else:
                    mediana = wyniki[grupa][pozycja]
            
            
            self.okno.ui.wyniki.insertRow(0)
            a=QTableWidgetItem(0)
            a.setText(grupa[0])
            self.okno.ui.wyniki.setItem(0,0,a)
            
            aa=QTableWidgetItem(0)            
            if grupa[1]!=None:
                aa.setText(grupa[1])
            else:
                aa.setText('-')
            self.okno.ui.wyniki.setItem(0,1,aa)
            
            aaa=QTableWidgetItem(0)
            if grupa[2]!=None:
                aaa.setText(grupa[2])
            else:
                aaa.setText('-')
            self.okno.ui.wyniki.setItem(0,2,aaa)
                        
            b=QTableWidgetItem(0)
            b.setText(`liczebnosc`)
            b.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,3,b)
            
            c=QTableWidgetItem(0)
            c.setText("%.2f" % suma)
            c.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,4,c)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % (srednia))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,5,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % min(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,6,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % max(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,7,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % wariancja)
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,8,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % odchylenie)
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,9,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % mediana)
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,10,d)
            
        self.okno.ui.wyniki.setSortingEnabled (1)
        
        for i in range(11):
            self.okno.ui.wyniki.resizeColumnToContents (i)



    
    def zerujPostep(self, nic):
        self.okno.ui.postep.setValue(0)
        self.okno.ui.wyniki.clearContents()
        
    def kopiowanie (self):
        if len(self.okno.ui.wyniki.selectedItems()) == 0:
            return
        
        schowek = QApplication.clipboard()
        tekst = ''
        liczbaKolumn = self.okno.ui.wyniki.columnCount()
        wierszPocz = self.okno.ui.wyniki.selectedRanges()[0].topRow()
        wierszKonc = self.okno.ui.wyniki.selectedRanges()[0].bottomRow()
        
        for i in range(liczbaKolumn):
            tekst = tekst + self.okno.ui.wyniki.horizontalHeaderItem(i).text()
            if i != (liczbaKolumn-1):
                    tekst = tekst + chr(9)
        tekst = tekst + chr(13)
        
        for i in range(wierszPocz, wierszKonc+1):
            for j in range(liczbaKolumn):
                tekst = tekst + self.okno.ui.wyniki.item(i,j).text()
                if j != (liczbaKolumn-1):
                    tekst = tekst + chr(9)
            if i != wierszKonc:
                tekst = tekst + chr(13)
                
        schowek.setText(tekst)

    def eksportCSV (self):
        
        u"Zapisuje zaznaczone wiersze do pliku CSV"
        
        if len(self.okno.ui.wyniki.selectedItems()) == 0:                       # Nie zaznaczono żadnego wiersza - wyjście
            return
        
        oknoPlikow = QFileDialog()                                              # Wybór pliku do zapisu
        oknoPlikow.setAcceptMode(1)
        if oknoPlikow.exec_() == 0:                                             # Nie wybrano żadnego pliku - wyjście
            return
        
        nazwaPliku = oknoPlikow.selectedFiles()[0]
        if not os.path.exists(nazwaPliku) and nazwaPliku[-4:].toUpper() != '.CSV':      # Jeżeli podano nieistniejący jeszcze plik to dodajemy rozszerzenie .csv
            nazwaPliku = nazwaPliku + '.csv'
        
        plik = open(nazwaPliku, 'wb')                                           # Otwarcie pliku do zapisu
        plikCSV = csv.writer( plik, delimiter=';' )
        
        tekst = []
        liczbaKolumn = self.okno.ui.wyniki.columnCount()
        wierszPocz = self.okno.ui.wyniki.selectedRanges()[0].topRow()
        wierszKonc = self.okno.ui.wyniki.selectedRanges()[0].bottomRow()
        
        for i in range(liczbaKolumn):                                           # Dodanie wiersza z nazwami kolumn
            tekst.append( self.okno.ui.wyniki.horizontalHeaderItem(i).text() )
        plikCSV.writerow( tekst )
        tekst = []
        
        for i in range( wierszPocz, wierszKonc+1 ):                             # Dodanie wierszy z danymi
            for j in range(liczbaKolumn):
                tekst.append( self.okno.ui.wyniki.item(i,j).text() )
            plikCSV.writerow( tekst )
            tekst = []
        
        plik.close()